﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Media3D;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 마우스 DOWN 처리하기 - OnMouseDown(e)

        /// <summary>
        /// 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            Point mousePoint = e.GetPosition(this.viewport3d);

            VisualTreeHelper.HitTest(this.viewport3d, null, HitTestFilter, new PointHitTestParameters(mousePoint));
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 히트 테스트 필터 처리하기 - HitTestFilter(hitTestResult)

        /// <summary>
        /// 히트 테스트 필터 처리하기
        /// </summary>
        /// <param name="hitTestResult">히트 테스트 결과</param>
        /// <returns>히트 테스트 결과 비헤이비어</returns>
        private HitTestResultBehavior HitTestFilter(HitTestResult hitTestResult)
        {
            RayMeshGeometry3DHitTestResult rayMeshGeometry3DHitTestResult = hitTestResult as RayMeshGeometry3DHitTestResult;

            if(rayMeshGeometry3DHitTestResult == null)
            {
                return HitTestResultBehavior.Continue;
            }

            ModelVisual3D modelVisual3D = rayMeshGeometry3DHitTestResult.VisualHit as ModelVisual3D;

            if(modelVisual3D == null)
            {
                return HitTestResultBehavior.Continue;
            }

            if(modelVisual3D == this.floorModelVisual3D)
            {
                GeometryModel3D geometryModel3D = rayMeshGeometry3DHitTestResult.ModelHit as GeometryModel3D;

                DiffuseMaterial diffuseMaterial = geometryModel3D.Material as DiffuseMaterial;

                Random random = new Random();

                diffuseMaterial.Brush = new SolidColorBrush
                (
                    Color.FromRgb
                    (
                        (byte)random.Next(256),
                        (byte)random.Next(256),
                        (byte)random.Next(256)
                    )
                );

                return HitTestResultBehavior.Stop;
            }

            if(modelVisual3D == this.tableModelVisual3D)
            {
                Storyboard storyboard = new Storyboard();

                DoubleAnimation doubleAnimation;

                #region 테이블을 위로 이동시키는 애니메이션을 정의한다.

                doubleAnimation = new DoubleAnimation();

                doubleAnimation.From = 0;
                doubleAnimation.To   = 2;

                Storyboard.SetTargetName(doubleAnimation, "tableTranslateTransform3D");
                Storyboard.SetTargetProperty(doubleAnimation, new PropertyPath(TranslateTransform3D.OffsetYProperty));

                storyboard.Children.Add(doubleAnimation);

                #endregion
                #region 테이블을 회전시키는 애니메이션을 정의한다.

                doubleAnimation = new DoubleAnimation();

                doubleAnimation.From      = 0;
                doubleAnimation.To        = 90;
                doubleAnimation.BeginTime = TimeSpan.FromSeconds(1);

                Storyboard.SetTargetName(doubleAnimation, "tableAxisAngleRotation3D");
                Storyboard.SetTargetProperty(doubleAnimation, new PropertyPath(AxisAngleRotation3D.AngleProperty));

                storyboard.Children.Add(doubleAnimation);

                #endregion
                #region 테이블을 아래로 이동시키는 애니메이션을 정의한다.

                doubleAnimation = new DoubleAnimation();

                doubleAnimation.From      = 2;
                doubleAnimation.To        = 0;
                doubleAnimation.BeginTime = TimeSpan.FromSeconds(2);

                Storyboard.SetTargetName(doubleAnimation, "tableTranslateTransform3D");
                Storyboard.SetTargetProperty(doubleAnimation, new PropertyPath(TranslateTransform3D.OffsetYProperty));

                storyboard.Children.Add(doubleAnimation);

                #endregion

                storyboard.Begin(this);

                return HitTestResultBehavior.Stop;
            }

            if(modelVisual3D == this.charModelVisual3D1 || modelVisual3D == this.charModelVisual3D2 || modelVisual3D == this.charModelVisual3D3 || modelVisual3D == this.charModelVisual3D4)
            {
                Transform3DGroup transform3DGroup = modelVisual3D.Transform as Transform3DGroup;

                if(transform3DGroup == null)
                {
                    return HitTestResultBehavior.Stop;
                }

                for(int i = 0; i < transform3DGroup.Children.Count; i++)
                {
                    TranslateTransform3D translateTransform3D = transform3DGroup.Children[i] as TranslateTransform3D;

                    if(translateTransform3D != null)
                    {
                        DoubleAnimation doubleAnimation = new DoubleAnimation();

                        DependencyProperty dependencyProperty = null;

                        if(translateTransform3D.OffsetZ == 0)
                        {
                            dependencyProperty = TranslateTransform3D.OffsetXProperty;

                            if(Math.Abs(translateTransform3D.OffsetX) < 2)
                            {
                                doubleAnimation.To = 2.5 * Math.Sign(translateTransform3D.OffsetX);
                            }
                            else
                            {
                                doubleAnimation.To = 1.25 * Math.Sign(translateTransform3D.OffsetX);
                            }
                        }
                        else
                        {
                            dependencyProperty = TranslateTransform3D.OffsetZProperty;

                            if(Math.Abs(translateTransform3D.OffsetZ) < 2)
                            {
                                doubleAnimation.To = 2.5 * Math.Sign(translateTransform3D.OffsetZ);
                            }
                            else
                            {
                                doubleAnimation.To = 1.25 * Math.Sign(translateTransform3D.OffsetZ);
                            }
                        }

                        translateTransform3D.BeginAnimation(dependencyProperty, doubleAnimation);

                        return HitTestResultBehavior.Stop;
                    }
                }
            }

            return HitTestResultBehavior.Continue;
        }

        #endregion
    }
}